@@ -1,6 +1,8 @@
 import logging
 
-from .design_matrices import design_matrices
+from .matrices import (
+    design_matrices,
+)
 from .model_description import model_description
 from .version import __version__
 
